<?php

declare(strict_types=1);

namespace Drupal\webex_client\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\webex_client\AuthServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Returns responses for webex routes.
 *
 * This controller handles the OAuth callback process for Webex, including
 * error handling and successful login responses. It is responsible for
 * redirecting users based on the outcome of the OAuth authentication process.
 */
final class OAuthCallbackController extends ControllerBase {

  /**
   * Constructs a new OAuthCallbackController object.
   *
   * @param \Drupal\webex_client\AuthServiceInterface $webexAuth
   *   The authentication service for Webex.
   */
  public function __construct(
    private readonly AuthServiceInterface $webexAuth,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('webex_client.auth'),
    );
  }

  /**
   * Builds the response for the OAuth callback.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The HTTP request object containing the query parameters.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response to the appropriate URL.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException
   */
  public function __invoke(Request $request): RedirectResponse {
    // Handle OAuth errors.
    if ($request->query->has('error')) {
      $this->getLogger('webex_client')->error('OAuth error: @error', [
        '@error' => $request->query->get('error_description', $request->query->get('error')),
      ]);

      $callback_url = Url::fromRoute('entity.webex.add_form', [], [
        'query' => ['error' => $request->query->get('error')],
      ])->toString();
      return new RedirectResponse($callback_url);
    }

    // Validate required parameters.
    $code = $request->query->get('code');
    $state = $request->query->get('state');

    // Process token exchange.
    if (!$this->webexAuth->accessToken($code, $state)) {
      $this->getLogger('webex_client')->error('Token exchange failed');
      $this->messenger()->addError(
        $this->t('Failed to authenticate with Webex. Please try again.')
      );
    }
    else {
      $this->getLogger('webex_client')->info('Authenticated with Webex.');
      $this->messenger()->addMessage(
        $this->t('Successfully authenticated with Webex.')
      );
    }

    return new RedirectResponse(
      Url::fromRoute('entity.webex.collection')->setAbsolute()->toString()
    );
  }

}
